'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };
// Components

// Helpers


require('../../../src/components/VDataTable/VDataTable.sass');

var _VData = require('../VData');

var _VBtn = require('../VBtn');

var _VBtn2 = _interopRequireDefault(_VBtn);

var _VDataIterator = require('../VDataIterator');

var _VDataIterator2 = _interopRequireDefault(_VDataIterator);

var _VDataTableHeader = require('./VDataTableHeader');

var _VDataTableHeader2 = _interopRequireDefault(_VDataTableHeader);

var _VDataTableVirtual = require('./VDataTableVirtual');

var _VDataTableVirtual2 = _interopRequireDefault(_VDataTableVirtual);

var _VIcon = require('../VIcon');

var _VIcon2 = _interopRequireDefault(_VIcon);

var _VProgressLinear = require('../VProgressLinear');

var _VProgressLinear2 = _interopRequireDefault(_VProgressLinear);

var _VRow = require('./VRow');

var _VRow2 = _interopRequireDefault(_VRow);

var _VRowGroup = require('./VRowGroup');

var _VRowGroup2 = _interopRequireDefault(_VRowGroup);

var _VSimpleCheckbox = require('../VCheckbox/VSimpleCheckbox');

var _VSimpleCheckbox2 = _interopRequireDefault(_VSimpleCheckbox);

var _helpers = require('../../util/helpers');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getPrefixedScopedSlots(prefix, scopedSlots) {
    return Object.keys(scopedSlots).filter(function (k) {
        return k.startsWith(prefix);
    }).reduce(function (obj, k) {
        obj[k.replace(prefix, '')] = scopedSlots[k];
        return obj;
    }, {});
}
/* @vue/component */
exports.default = _VDataIterator2.default.extend({
    name: 'v-data-table',
    provide: function provide() {
        return { dataTable: this };
    },

    props: {
        headers: {
            type: Array
        },
        showSelect: Boolean,
        showExpand: Boolean,
        showGroupBy: Boolean,
        virtualRows: Boolean,
        mobileBreakpoint: {
            type: String,
            default: 'sm'
        },
        height: [Number, String],
        hideDefaultFooter: Boolean,
        hideDefaultHeader: Boolean,
        caption: String,
        dense: Boolean,
        footerProps: Object,
        headerProps: Object,
        calculateWidths: Boolean
    },
    data: function data() {
        return {
            internalGroupBy: [],
            openCache: {},
            widths: []
        };
    },

    computed: {
        computedHeaders: function computedHeaders() {
            var _this = this;

            var headers = this.headers.filter(function (h) {
                return h.value === undefined || !_this.internalGroupBy.find(function (v) {
                    return v === h.value;
                });
            });
            this.showSelect && headers.unshift({ text: '', value: 'dataTableSelect', sortable: false, width: '1px' });
            this.showExpand && headers.unshift({ text: '', value: 'dataTableExpand', sortable: false, width: '1px' });
            return headers;
        },
        isMobile: function isMobile() {
            return !!this.$vuetify.breakpoint[this.mobileBreakpoint]; // TODO: fix typing
        }
    },
    mounted: function mounted() {
        // if ((!this.sortBy || !this.sortBy.length) && (!this.options.sortBy || !this.options.sortBy.length)) {
        //   const firstSortable = this.headers.find(h => !('sortable' in h) || !!h.sortable)
        //   if (firstSortable) this.updateOptions({ sortBy: [firstSortable.value], sortDesc: [false] })
        // }
        if (this.calculateWidths) {
            window.addEventListener('resize', this.calcWidths);
        }
        this.calcWidths();
    },
    beforeDestroy: function beforeDestroy() {
        if (this.calculateWidths) {
            window.removeEventListener('resize', this.calcWidths);
        }
    },

    methods: {
        calcWidths: function calcWidths() {
            this.widths = Array.from(this.$el.querySelectorAll('th')).map(function (e) {
                return e.clientWidth;
            });
        },
        customFilterWithColumns: function customFilterWithColumns(items, search) {
            var _this2 = this;

            var filterableHeaders = this.headers.filter(function (h) {
                return !!h.filter;
            });
            if (filterableHeaders.length) {
                items = items.filter(function (i) {
                    return filterableHeaders.every(function (h) {
                        return h.filter((0, _helpers.getObjectValueByPath)(i, h.value), _this2.search, i);
                    });
                });
            }
            return this.customFilter(items, search);
        },
        customSortWithHeaders: function customSortWithHeaders(items, sortBy, sortDesc, locale) {
            var customSorters = this.computedHeaders.reduce(function (acc, header) {
                if (header.sort) acc[header.value] = header.sort;
                return acc;
            }, {});
            return this.customSort(items, sortBy, sortDesc, locale, customSorters);
        },
        createItemProps: function createItemProps(item) {
            var props = _VDataIterator2.default.options.methods.createItemProps.call(this, item);
            // TODO: Will {}, props mess up .syncing?
            return Object.assign(props, { headers: this.computedHeaders });
        },
        genCaption: function genCaption(props) {
            if (this.caption) return [this.$createElement('caption', [this.caption])];
            return this.genSlots('caption', props);
        },
        genColgroup: function genColgroup(props) {
            var _this3 = this;

            return this.$createElement('colgroup', this.computedHeaders.map(function (header) {
                return _this3.$createElement('col', {
                    class: {
                        'divider': header.divider || header.resizable,
                        'resizable': header.resizable
                    },
                    style: {
                        width: header.width
                    }
                });
            }));
        },
        genLoading: function genLoading() {
            var progress = this.$slots['progress'] ? this.$slots.progress : this.$createElement(_VProgressLinear2.default, {
                props: {
                    color: this.loading === true ? 'primary' : this.loading,
                    height: 2,
                    indeterminate: true
                }
            });
            var th = this.$createElement('th', {
                staticClass: 'column',
                attrs: {
                    colspan: this.computedHeaders.length
                }
            }, [progress]);
            return this.$createElement('tr', {
                staticClass: 'v-data-table__progress'
            }, [th]);
        },
        genHeaders: function genHeaders(props) {
            var data = {
                props: _extends({}, this.headerProps, {
                    headers: this.computedHeaders,
                    options: props.options,
                    mobile: this.isMobile
                }),
                on: {
                    sort: props.sort,
                    group: props.group
                }
            };
            var children = [this.genSlots('header', data)];
            if (!this.hideDefaultHeader) {
                var scopedSlots = getPrefixedScopedSlots('header.', this.$scopedSlots);
                children.push(this.$createElement(_VDataTableHeader2.default, _extends({}, data, {
                    scopedSlots: scopedSlots
                })));
            }
            if (this.loading) children.push(this.genLoading());
            return children;
        },
        genEmptyWrapper: function genEmptyWrapper(content) {
            return this.$createElement('tr', [this.$createElement('td', {
                attrs: {
                    colspan: this.computedHeaders.length
                }
            }, content)]);
        },
        genItems: function genItems(items, props) {
            var empty = this.genEmpty(props.pagination.itemsLength);
            if (empty) return [empty];
            return props.options.groupBy.length ? this.genGroupedRows(props.groupedItems, props) : this.genRows(items, props);
        },
        genGroupedRows: function genGroupedRows(groupedItems, props) {
            var _this4 = this;

            var groups = Object.keys(groupedItems || {});
            return groups.map(function (group) {
                if (!_this4.openCache.hasOwnProperty(group)) _this4.$set(_this4.openCache, group, true);
                if (_this4.$scopedSlots.group) {
                    return _this4.$scopedSlots.group({
                        group: group,
                        options: props.options,
                        items: groupedItems[group],
                        headers: _this4.computedHeaders
                    });
                } else {
                    return _this4.genDefaultGroupedRow(group, groupedItems[group], props);
                }
            });
        },
        genDefaultGroupedRow: function genDefaultGroupedRow(group, items, props) {
            var _this5 = this;

            var isOpen = !!this.openCache[group];
            var children = [this.$createElement('template', { slot: 'row.content' }, this.genDefaultRows(props.groupedItems[group], props))];
            if (this.$scopedSlots['group.header']) {
                children.unshift(this.$createElement('template', { slot: 'column.header' }, [this.$scopedSlots['group.header']({ group: group, groupBy: props.options.groupBy, items: items, headers: this.computedHeaders })]));
            } else {
                var toggle = this.$createElement(_VBtn2.default, {
                    staticClass: 'ma-0',
                    props: {
                        icon: true,
                        small: true
                    },
                    on: {
                        click: function click() {
                            return _this5.$set(_this5.openCache, group, !_this5.openCache[group]);
                        }
                    }
                }, [this.$createElement(_VIcon2.default, [isOpen ? 'remove' : 'add'])]);
                var remove = this.$createElement(_VBtn2.default, {
                    staticClass: 'ma-0',
                    props: {
                        icon: true,
                        small: true
                    },
                    on: {
                        click: function click() {
                            return props.updateOptions({ groupBy: [], groupDesc: [] });
                        }
                    }
                }, [this.$createElement(_VIcon2.default, ['close'])]);
                var column = this.$createElement('td', {
                    staticClass: 'text-xs-left',
                    attrs: {
                        colspan: this.computedHeaders.length
                    }
                }, [toggle, props.options.groupBy[0] + ': ' + group, remove]);
                children.unshift(this.$createElement('template', { slot: 'column.header' }, [column]));
            }
            if (this.$scopedSlots['group.summary']) {
                children.push(this.$createElement('template', { slot: 'column.summary' }, [this.$scopedSlots['group.summary']({ group: group, groupBy: props.options.groupBy, items: items, headers: this.computedHeaders })]));
            }
            return this.$createElement(_VRowGroup2.default, {
                key: group,
                props: {
                    value: isOpen
                }
            }, children);
        },
        genRows: function genRows(items, props) {
            return this.$scopedSlots.item ? this.genScopedRows(items, props) : this.genDefaultRows(items, props);
        },
        genScopedRows: function genScopedRows(items, props) {
            var _this6 = this;

            return items.map(function (item) {
                return _this6.$scopedSlots.item(_this6.createItemProps(item));
            });
        },
        genDefaultRows: function genDefaultRows(items, props) {
            var _this7 = this;

            return this.$scopedSlots['item.expanded'] ? items.map(function (item) {
                return _this7.genDefaultExpandedRow(item);
            }) : items.map(function (item) {
                return _this7.genDefaultSimpleRow(item);
            });
        },
        genDefaultExpandedRow: function genDefaultExpandedRow(item) {
            var isExpanded = this.isExpanded(item);
            var headerRow = this.genDefaultSimpleRow(item, isExpanded ? 'expanded expanded__row' : null);
            var expandedRow = this.$createElement('tr', {
                staticClass: 'expanded expanded__content'
            }, [this.$scopedSlots['item.expanded']({ item: item, headers: this.computedHeaders })]);
            return this.$createElement(_VRowGroup2.default, {
                props: {
                    value: isExpanded
                }
            }, [this.$createElement('template', { slot: 'row.header' }, [headerRow]), this.$createElement('template', { slot: 'row.content' }, [expandedRow])]);
        },
        genDefaultSimpleRow: function genDefaultSimpleRow(item) {
            var _this8 = this;

            var classes = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

            var scopedSlots = getPrefixedScopedSlots('item.', this.$scopedSlots);
            if (this.showSelect) {
                var data = {
                    props: {
                        value: this.isSelected(item)
                    },
                    on: {
                        input: function input(v) {
                            return _this8.select(item, v);
                        }
                    }
                };
                var slot = scopedSlots['dataTableSelect'];
                scopedSlots['dataTableSelect'] = slot ? function () {
                    return slot(data);
                } : function () {
                    return _this8.$createElement(_VSimpleCheckbox2.default, _extends({
                        staticClass: 'v-data-table__checkbox'
                    }, data));
                };
            }
            var expanded = this.isExpanded(item);
            if (this.showExpand) {
                scopedSlots['dataTableExpand'] = function () {
                    return _this8.$createElement(_VIcon2.default, {
                        staticClass: 'expand__icon',
                        class: {
                            'expand__icon--active': expanded
                        },
                        on: {
                            click: function click() {
                                return _this8.expand(item, !expanded);
                            }
                        }
                    }, [_this8.$vuetify.icons.expand]);
                }; // TODO: prop?
            }
            return this.$createElement(_VRow2.default, {
                key: (0, _helpers.getObjectValueByPath)(item, this.itemKey),
                class: classes,
                props: {
                    headers: this.computedHeaders,
                    item: item,
                    mobile: this.isMobile,
                    rtl: this.$vuetify.rtl
                },
                scopedSlots: scopedSlots
            });
        },
        genBody: function genBody(props) {
            if (this.$scopedSlots.body) return this.$scopedSlots.body(props);
            return this.$createElement('tbody', [this.genSlots('body.prepend', props), this.genItems(props.items, props), this.genSlots('body.append', props)]);
        },
        genFooters: function genFooters(props) {
            var data = {
                props: _extends({}, this.footerProps, {
                    options: props.options,
                    pagination: props.pagination
                }),
                on: {
                    'update:options': function updateOptions(value) {
                        return props.updateOptions(value);
                    }
                },
                widths: this.widths,
                headers: this.computedHeaders
            };
            var children = [this.genSlots('footer', data)];
            if (!this.hideDefaultFooter) {
                children.push(this.$createElement(_VData.VDataFooter, data));
            }
            return children;
        },
        genTable: function genTable(props) {
            return this.$createElement('div', {
                staticClass: 'v-data-table__wrapper',
                style: {
                    height: (0, _helpers.convertToUnit)(this.height)
                }
            }, [this.$createElement('table', [this.genCaption(props), this.genColgroup(props), this.genHeaders(props), this.genBody(props)])]);
        },
        genDefaultScopedSlot: function genDefaultScopedSlot(props) {
            var _this9 = this;

            var classes = _extends({
                'v-data-table--dense': this.dense,
                'v-data-table--fixed': !!this.height,
                'v-data-table--mobile': this.isMobile
            }, this.themeClasses);
            // TODO: Do we have to support static? Is there another way?
            // if (this.static) {
            //   return this.$createElement('div', ['static'])
            // }
            if (this.virtualRows) {
                return this.$createElement(_VDataTableVirtual2.default, {
                    class: classes,
                    props: {
                        itemsLength: props.items.length,
                        height: Number(this.height)
                    },
                    scopedSlots: {
                        items: function items(_ref) {
                            var start = _ref.start,
                                stop = _ref.stop;
                            return _this9.genItems(props.items.slice(start, stop), props);
                        } // TODO: What is up with this typing?
                    }
                }, [this.$createElement('template', { slot: 'caption' }, this.genCaption(props)), this.$createElement('template', { slot: 'header' }, this.genHeaders(props)), this.$createElement('template', { slot: 'footer' }, this.genFooters(props))]);
            }
            return this.$createElement('div', {
                staticClass: 'v-data-table',
                class: classes
            }, [this.genSlots('top'), this.genTable(props), this.genFooters(props)]);
        }
    },
    render: function render() {
        var _this10 = this;

        return this.$createElement(_VData.VData, {
            props: _extends({}, this.$props, {
                customFilter: this.customFilterWithColumns,
                customSort: this.customSortWithHeaders
            }),
            on: {
                'update:options': function updateOptions(v, old) {
                    _this10.internalGroupBy = v.groupBy || [];
                    !(0, _helpers.deepEqual)(v, old) && _this10.$emit('update:options', v);
                },
                'update:page': function updatePage(v) {
                    return _this10.$emit('update:page', v);
                },
                'update:itemsPerPage': function updateItemsPerPage(v) {
                    return _this10.$emit('update:itemsPerPage', v);
                },
                'update:sortBy': function updateSortBy(v) {
                    return _this10.$emit('update:sortBy', v);
                },
                'update:sortDesc': function updateSortDesc(v) {
                    return _this10.$emit('update:sortDesc', v);
                },
                'update:groupBy': function updateGroupBy(v) {
                    return _this10.$emit('update:groupBy', v);
                },
                'update:groupDesc': function updateGroupDesc(v) {
                    return _this10.$emit('update:groupDesc', v);
                },
                'pagination': function pagination(v, old) {
                    return !(0, _helpers.deepEqual)(v, old) && _this10.$emit('pagination', v);
                },
                'current-items': function currentItems(v) {
                    _this10.internalCurrentItems = v;
                    _this10.$emit('current-items', v);
                },
                'pageCount': function pageCount(v) {
                    return _this10.$emit('pageCount', v);
                }
            },
            scopedSlots: {
                default: this.genDefaultScopedSlot
            }
        });
    }
});
//# sourceMappingURL=VDataTable.js.map