'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _vue = require('vue');

var _vue2 = _interopRequireDefault(_vue);

var _helpers = require('../../util/helpers');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = _vue2.default.extend({
    name: 'v-data-table-virtual',
    props: {
        bufferLength: {
            type: Number,
            default: 10
        },
        headerHeight: {
            type: Number,
            default: 48
        },
        height: Number,
        itemsLength: Number,
        rowHeight: {
            type: Number,
            default: 48
        }
    },
    data: function data() {
        return {
            scrollTop: 0
        };
    },
    computed: {
        bufferHeight: function bufferHeight() {
            return this.bufferLength * this.rowHeight / 2;
        },
        totalHeight: function totalHeight() {
            var headerHeight = this.$slots.header ? this.headerHeight : 0;
            return this.itemsLength * this.rowHeight + headerHeight;
        },
        startIndex: function startIndex() {
            return Math.max(0, Math.ceil((this.scrollTop - this.bufferHeight) / this.rowHeight));
        },
        offsetTop: function offsetTop() {
            return Math.max(0, this.startIndex * this.rowHeight);
        },
        visibleRows: function visibleRows() {
            return Math.ceil(Number(this.height) / this.rowHeight) + this.bufferLength;
        },
        stopIndex: function stopIndex() {
            return this.startIndex + this.visibleRows;
        },
        offsetBottom: function offsetBottom() {
            return Math.max(0, (this.itemsLength - this.visibleRows) * this.rowHeight - this.offsetTop);
        }
    },
    watch: {
        scrollTop: 'setScrollTop'
    },
    mounted: function mounted() {
        this.setScrollTop();
    },

    methods: {
        createStyleHeight: function createStyleHeight(height) {
            return {
                height: height + 'px'
            };
        },
        setScrollTop: function setScrollTop() {
            var wrapper = this.$refs.wrapper;
            wrapper.scrollTop = this.scrollTop;
        },
        genScroller: function genScroller() {
            var _this = this;

            return this.$createElement('div', {
                ref: 'scroller',
                staticClass: 'v-data-table-virtual__scroller',
                on: {
                    scroll: function scroll(e) {
                        var target = e.target;
                        _this.scrollTop = target.scrollTop;
                    }
                }
            }, [this.$createElement('div', {
                style: this.createStyleHeight(this.totalHeight)
            })]);
        },
        genBody: function genBody() {
            return this.$createElement('tbody', [this.$createElement('tr', { style: this.createStyleHeight(this.offsetTop) }), this.$scopedSlots.items ? this.$scopedSlots.items({ start: this.startIndex, stop: this.stopIndex }) : null, this.$createElement('tr', { style: this.createStyleHeight(this.offsetBottom) })]);
        },
        genWrapper: function genWrapper() {
            var _this2 = this;

            return this.$createElement('div', {
                ref: 'wrapper',
                staticClass: 'v-data-table-virtual__wrapper',
                on: {
                    mousewheel: function mousewheel(e) {
                        var scroller = _this2.$refs.scroller;
                        scroller.scrollTop = Math.max(0, Math.min(_this2.totalHeight, _this2.scrollTop + e.deltaY));
                    }
                }
            }, [this.$createElement('table', [this.$slots.caption, this.$slots.header, this.genBody()])]);
        }
    },
    render: function render(h) {
        return h('div', {
            staticClass: 'v-data-table v-data-table-virtual',
            style: {
                height: (0, _helpers.convertToUnit)(this.height)
            }
        }, [this.genWrapper(), this.genScroller(), this.$slots.footer]);
    }
});
//# sourceMappingURL=VDataTableVirtual.js.map