var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

// Components
import { VData } from '../VData';
// Helpers
import { deepEqual, getObjectValueByPath } from '../../util/helpers';
import mixins from '../../util/mixins';
import Themeable from '../../mixins/themeable';
/* @vue/component */
export default mixins(Themeable).extend({
    name: 'v-data-iterator',
    props: _extends({}, VData.options.props, {
        value: {
            type: Array,
            default: function _default() {
                return [];
            }
        },
        singleSelect: Boolean,
        expanded: {
            type: Array,
            default: function _default() {
                return [];
            }
        },
        singleExpand: Boolean,
        loading: [Boolean, String],
        noResultsText: {
            type: String,
            default: '$vuetify.dataIterator.noResultsText'
        },
        noDataText: {
            type: String,
            default: '$vuetify.noDataText'
        },
        loadingText: {
            type: String,
            default: '$vuetify.dataIterator.loadingText'
        }
    }),
    data: function data() {
        return {
            selection: {},
            expansion: {},
            internalCurrentItems: []
        };
    },
    computed: {
        everyItem: function everyItem() {
            var _this = this;

            return !!this.internalCurrentItems.length && this.internalCurrentItems.every(function (i) {
                return _this.isSelected(i);
            });
        },
        someItems: function someItems() {
            var _this2 = this;

            return this.internalCurrentItems.some(function (i) {
                return _this2.isSelected(i);
            });
        }
    },
    watch: {
        value: function value(_value) {
            var _this3 = this;

            this.selection = _value.reduce(function (selection, item) {
                selection[getObjectValueByPath(item, _this3.itemKey)] = true;
                return selection;
            }, {});
        },

        selection: {
            handler: function handler(value, old) {
                var _this4 = this;

                if (deepEqual(value, old)) return;
                var keys = Object.keys(value).filter(function (k) {
                    return value[k];
                });
                var selected = !keys.length ? [] : this.items.filter(function (i) {
                    return keys.includes(String(getObjectValueByPath(i, _this4.itemKey)));
                });
                this.$emit('input', selected);
            },

            deep: true
        },
        expanded: function expanded(value) {
            var _this5 = this;

            this.expansion = value.reduce(function (expansion, item) {
                expansion[getObjectValueByPath(item, _this5.itemKey)] = true;
                return expansion;
            }, {});
        },
        expansion: function expansion(value, old) {
            var _this6 = this;

            if (deepEqual(value, old)) return;
            var keys = Object.keys(value).filter(function (k) {
                return value[k];
            });
            var expanded = !keys.length ? [] : this.items.filter(function (i) {
                return keys.includes(String(getObjectValueByPath(i, _this6.itemKey)));
            });
            this.$emit('update:expanded', expanded);
        }
    },
    methods: {
        toggleSelectAll: function toggleSelectAll(value) {
            var _this7 = this;

            this.internalCurrentItems.forEach(function (item) {
                var key = getObjectValueByPath(item, _this7.itemKey);
                _this7.$set(_this7.selection, key, value);
            });
        },
        isSelected: function isSelected(item) {
            return this.selection[getObjectValueByPath(item, this.itemKey)] || false;
        },
        select: function select(item) {
            var value = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

            var selection = this.singleSelect ? {} : Object.assign({}, this.selection);
            var key = getObjectValueByPath(item, this.itemKey);
            if (value) selection[key] = true;else delete selection[key];
            this.selection = selection;
            this.$emit('item-selected', { item: item, value: value });
        },
        isExpanded: function isExpanded(item) {
            return this.expansion[getObjectValueByPath(item, this.itemKey)] || false;
        },
        expand: function expand(item) {
            var value = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

            var expansion = this.singleExpand ? {} : Object.assign({}, this.expansion);
            var key = getObjectValueByPath(item, this.itemKey);
            if (value) expansion[key] = true;else delete expansion[key];
            this.expansion = expansion;
            this.$emit('item-expanded', { item: item, value: value });
        },
        createItemProps: function createItemProps(item) {
            var _this8 = this;

            var props = {
                item: item,
                select: {
                    props: {
                        value: this.isSelected(item)
                    },
                    on: {
                        input: function input(v) {
                            return _this8.select(item, v);
                        }
                    }
                },
                expand: {
                    props: {
                        value: this.isExpanded(item)
                    },
                    on: {
                        input: function input(v) {
                            return _this8.expand(item, v);
                        }
                    }
                }
            };
            return props;
        },
        genEmptyWrapper: function genEmptyWrapper(content) {
            return this.$createElement('div', content);
        },
        genEmpty: function genEmpty(itemsLength) {
            if (itemsLength <= 0 && this.loading) {
                var loading = this.$slots['loading'] || this.$vuetify.lang.t(this.loadingText);
                return this.genEmptyWrapper(loading);
            } else if (itemsLength <= 0 && !this.items.length) {
                var noData = this.$slots['no-data'] || this.$vuetify.lang.t(this.noDataText);
                return this.genEmptyWrapper(noData);
            } else if (itemsLength <= 0 && this.search) {
                var noResults = this.$slots['no-results'] || this.$vuetify.lang.t(this.noResultsText);
                return this.genEmptyWrapper(noResults);
            }
            return null;
        },
        genItems: function genItems(props) {
            var _this9 = this;

            var empty = this.genEmpty(props.pagination.itemsLength);
            if (empty) return [empty];
            if (this.$scopedSlots.default) return this.$scopedSlots.default(props);
            if (this.$scopedSlots.item) {
                return props.items.map(function (item) {
                    return _this9.$scopedSlots.item(_this9.createItemProps(item));
                });
            }
            return [];
        },
        genSlots: function genSlots(slot) {
            var props = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

            if (this.$scopedSlots[slot]) return this.$scopedSlots[slot](props);else if (this.$slots[slot]) return this.$slots[slot];
            return [];
        },
        genDefaultScopedSLot: function genDefaultScopedSLot(props) {
            return this.$createElement('div', {
                staticClass: 'v-data-iterator'
            }, [this.genSlots('prepend', props), this.genItems(props), this.genSlots('append', props)]);
        }
    },
    render: function render() {
        var _this10 = this;

        return this.$createElement(VData, {
            props: this.$props,
            on: {
                'update:options': function updateOptions(v, old) {
                    return !deepEqual(v, old) && _this10.$emit('update:options', v);
                },
                'update:page': function updatePage(v) {
                    return _this10.$emit('update:page', v);
                },
                'update:itemsPerPage': function updateItemsPerPage(v) {
                    return _this10.$emit('update:itemsPerPage', v);
                },
                'update:sortBy': function updateSortBy(v) {
                    return _this10.$emit('update:sortBy', v);
                },
                'update:sortDesc': function updateSortDesc(v) {
                    return _this10.$emit('update:sortDesc', v);
                },
                'update:groupBy': function updateGroupBy(v) {
                    return _this10.$emit('update:groupBy', v);
                },
                'update:groupDesc': function updateGroupDesc(v) {
                    return _this10.$emit('update:groupDesc', v);
                },
                'pagination': function pagination(v, old) {
                    return !deepEqual(v, old) && _this10.$emit('pagination', v);
                },
                'current-items': function currentItems(v) {
                    _this10.internalCurrentItems = v;
                    _this10.$emit('current-items', v);
                }
            },
            scopedSlots: {
                default: this.genDefaultScopedSLot
            }
        });
    }
});
//# sourceMappingURL=VDataIterator.js.map